from pylusat.geotools import (select_by_location, erase,
                              within_dist, spatial_join)
from pylusat.base import GeoDataFrameManager

import geopandas as gpd
import pandas as pd
import numpy as np

from pylusat._version import __version__



from rasterstats import zonal_stats
import pandas as pd
from affine import Affine
from geopandas import GeoDataFrame
from pylusat.base import GeoDataFrameManager
from pylusat.utils import read_raster


def _to_affine(cellsize, max_y, min_x):
    """Construct an Affine object."""
    return Affine(cellsize, 0, min_x, 0, -cellsize, max_y)


def zonal_stats_raster(zone_gdf, raster, stats=None, nodata=None):
    """
    Return specified stats for each geometry in the zone GeoDataFrame.

    Parameters
    ----------
    zone_gdf : GeoDataFrame
        The zone GeoDataFrame whose geometry must be polygon.
    raster : str
        A path to a tif file or a connection string to a raster on PostgreSQL.
        The raster dataset whose values are summarized.
    stats : list of str, or space-delimited str, optional
        Which statistics to calculate for each zone.
        Defaults to rasterstats.utils.DEFAULT_STATS which is
        ['count', 'min', 'max', 'mean'].
        Other valid stats are ['sum', 'std', 'median', 'majority', 'minority',
        'unique', 'range', 'nodata', 'nan'].
        See rasterstats for more details.
    nodata : int or float
        Value for no data cells.

    Returns
    -------
    output_gdf : GeoDataFrame
         Returns a GeoDataFrame containing new column(s) for each type of
         statistic specified in the ``stats`` argument.
    """
    if not GeoDataFrameManager(zone_gdf).geom_type_validate("Polygon"):
        raise ValueError("zone GeoDataFrame must be polygon.")

    raster_arr, cellsize, max_y, min_x, nodata = read_raster(raster, nodata)
    affine = _to_affine(cellsize, max_y, min_x)
    zonal_output = zonal_stats(vectors=zone_gdf.geometry, raster=raster_arr,
                               nodata=nodata, affine=affine, stats=stats,
                               all_touched=True)
    if zone_gdf.index.name is None:
        zone_index = 'index'
    else:
        zone_index = zone_gdf.index.name

    zone_gdf.reset_index(inplace=True)
    output_gdf = zone_gdf.join(pd.DataFrame(zonal_output))
    output_gdf.set_index(zone_index, inplace=True)
    return output_gdf



import os
import geopandas as gpd
from pylusat.interpolate import idw
import unittest


class TestInterpolate(unittest.TestCase):

    schools = "schools"     # point geometry
    acs2016 = "acs2016"     # polygon geometry
    dataset_path = os.path.join(os.path.dirname(os.getcwd()), "datasets")

    schools_shp = os.path.join(dataset_path, schools, f'{schools}.shp')
    acs2016_shp = os.path.join(dataset_path, acs2016, f'{acs2016}.shp')

    @classmethod
    def setUpClass(cls):
        cls.schools_gdf = gpd.read_file(cls.schools_shp)
        cls.acs2016_gdf = gpd.read_file(cls.acs2016_shp)

    def test_idw(self):
        idw(self.acs2016_gdf, self.schools_gdf, 'ENROLLMENT',
            power=2.00, n_neighbor=12)


if __name__ == "__main__":
    unittest.main()



import os
import geopandas as gpd
from pylusat.density import of_point, of_line
import unittest


class TestDensity(unittest.TestCase):

    schools = r"schools\schools.shp"     # point geometry
    highway = r"highway\highway.shp"     # line geometry
    acs2016 = r"acs2016\acs2016.shp"     # polygon geometry
    habitat = r"habitat\habitat.tif"     # raster data (tiff)
    dataset_path = os.path.join(os.path.dirname(os.getcwd()), "datasets")

    schools_shp = os.path.join(dataset_path, schools)
    highway_shp = os.path.join(dataset_path, highway)
    acs2016_shp = os.path.join(dataset_path, acs2016)
    habitat_tif = os.path.join(dataset_path, habitat)

    schools_enrollment = 'ENROLLMENT'

    @classmethod
    def setUpClass(cls):
        cls.schools_gdf = gpd.read_file(cls.schools_shp)
        cls.highway_gdf = gpd.read_file(cls.highway_shp)
        cls.acs2016_gdf = gpd.read_file(cls.acs2016_shp)

    def test_of_point_no_search(self):
        of_point(self.acs2016_gdf, self.schools_gdf)

    def test_of_point_search(self):
        of_point(self.acs2016_gdf, self.schools_gdf, '1 mile', 'acre',
                 self.schools_enrollment)

    def test_of_line(self):
        of_line(self.acs2016_gdf, self.highway_gdf, search_radius='1 mile')


if __name__ == "__main__":
    unittest.main()
